"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PdfData = void 0;
const pdf_1 = require("pdfjs-dist/es5/build/pdf");
class PdfData {
    constructor(pages, text, fingerprint, outline, metaData) {
        this.pages = pages;
        this.text = text;
        this.fingerprint = fingerprint;
        this.outline = outline;
        if (metaData != null) {
            this.info = metaData.info;
            this.metadata = metaData.metadata;
        }
    }
    static async getPageNumber(pdf_document, pageRef, cache) {
        const ref = pageRef.gen === 0 ? `${pageRef.num}R` : `${pageRef.num}R${pageRef.gen}`;
        let number = cache[ref];
        if (number == null) {
            number = await pdf_document.getPageIndex(pageRef);
            cache[ref] = number;
        }
        return number;
    }
    static async parseOutline(pdf_document, outlineData, cache) {
        const outline = [];
        for (const o of outlineData) {
            let dest = o.dest ? o.dest[0] : null;
            if (dest == null) {
                if (o.unsafeUrl != null) {
                    outline.push({
                        title: o.title,
                        url: o.unsafeUrl,
                        childs: o.items ? await PdfData.parseOutline(pdf_document, o.items, cache) : undefined
                    });
                }
            }
            else {
                outline.push({
                    title: o.title,
                    page: await PdfData.getPageNumber(pdf_document, dest, cache),
                    childs: o.items ? await PdfData.parseOutline(pdf_document, o.items, cache) : undefined
                });
            }
        }
        return outline;
    }
    /**
     * Returns the extracted data from a pdf file
     *
     * @param {Uint8Array} data The pdf file in the form of byte data
     * @param {PdfDataOptions} options The options how to read the data
     * @returns {Promise<PdfData>}
     * @memberof PdfData
     */
    static async extract(data, options = {}) {
        const pdf_document = await pdf_1.getDocument({
            data: data,
            password: options.password,
            verbosity: pdf_1.VerbosityLevel.ERRORS
        }).promise;
        const counter = options.max && options.max > 0 ?
            (options.max > pdf_document.numPages ?
                pdf_document.numPages
                : options.max)
            : pdf_document.numPages;
        const sort = !!options.sort;
        const text_array = [];
        const pages = [];
        for (let i = 1; i <= counter; i++) {
            const page = await pdf_document.getPage(i).catch(_ => null);
            pages[i] = page;
            const pageText = page == null ? '' : await page.getTextContent().then(textContent => {
                /*
                    transform is a array with a transform matrix [scale x,shear x,shear y,scale y,offset x, offset y]

                    0,1         1,1
                      -----------
                      |         |
                      |         |
                      |   pdf   |
                      |         |
                      |         |
                      -----------
                    0,0         1,0
                */
                //coordinate based sorting
                if (sort)
                    textContent.items.sort((e1, e2) => {
                        if (e1.transform[5] < e2.transform[5])
                            return 1;
                        else if (e1.transform[5] > e2.transform[5])
                            return -1;
                        else if (e1.transform[4] < e2.transform[4])
                            return -1;
                        else if (e1.transform[4] > e2.transform[4])
                            return 1;
                        else
                            return 0;
                    });
                let lastLineY, text = '';
                for (let item of textContent.items) {
                    if (!lastLineY || lastLineY == item.transform[5]) {
                        text += item.str;
                    }
                    else {
                        text += '\n' + item.str;
                    }
                    lastLineY = item.transform[5];
                }
                return text;
            }, _ => '');
            text_array.push(pageText);
        }
        const metaData = await pdf_document.getMetadata().catch(_ => null);
        const outline = await this.parseOutline(pdf_document, await pdf_document.getOutline(), {});
        pdf_document.destroy();
        return new PdfData(pdf_document.numPages, text_array, pdf_document.fingerprint, outline, metaData);
    }
}
exports.PdfData = PdfData;
//# sourceMappingURL=index.js.map